"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
Object.defineProperty(exports, "PortalView", {
  enumerable: true,
  get: function () {
    return _PortalView.default;
  }
});
exports.unsubscribe = exports.syncSome = exports.syncOne = exports.syncAll = exports.subscribe = exports.register = exports.publish = exports.getPortal = exports.enableSecureLiveUpdates = exports.addPortals = exports.addPortal = void 0;

var _reactNative = require("react-native");

var _PortalView = _interopRequireDefault(require("./PortalView"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const {
  IONPortalPubSub,
  IONPortalsReactNative
} = _reactNative.NativeModules;
const PortalsPubSub = new _reactNative.NativeEventEmitter(IONPortalPubSub);
const subscriptionMap = new Map();
/**
 * Subscribes to messages for a topic
 *
 * @param topic The topic to subscribe to
 * @param onMessageReceived The callback to invoke when a message is received
 * @returns A Promise<number> containing the unique subscription reference. This will need to be stored for calling {@link unsubscribe}.
 */

const subscribe = async (topic, onMessageReceived) => {
  const subscriptionRef = await IONPortalPubSub.subscribe(topic);
  const subscriber = PortalsPubSub.addListener('PortalsSubscription', message => {
    if (message.subscriptionRef === subscriptionRef) {
      onMessageReceived(message);
    }
  });
  subscriptionMap.set(subscriptionRef, subscriber);
  return subscriptionRef;
};
/**
 * Unsubscribes from events for the provided topic and subscription reference
 *
 * @param topic The topic to unsubscribe from
 * @param subRef The unique subscription reference received when initially calling {@link subscribe}
 */


exports.subscribe = subscribe;

const unsubscribe = (topic, subRef) => {
  IONPortalPubSub.unsubscribe(topic, subRef);
  const subscription = subscriptionMap.get(subRef);

  if (subscription !== undefined) {
    subscription.remove();
    subscriptionMap.delete(subRef);
  }
};
/**
 * Publishes a message to the provided topic
 *
 * @param topic The topic to publish the message to
 * @param data The data to publish to subscribers
 */


exports.unsubscribe = unsubscribe;

const publish = (topic, data) => {
  const msg = {
    message: data
  };
  IONPortalPubSub.publish(topic, msg);
};
/**
 * Validates that a valid registration key has been procured from http://ionic.io/register-portals
 * @param key The registration key
 * @returns Promise<void>
 */


exports.publish = publish;

const register = async key => {
  return IONPortalsReactNative.register(key);
};
/**
 * The configuration of a web application to be embedded in a React Native application.
 */


exports.register = register;

/**
 * Adds a Portal to an internal registry. Must be called before attempting to render a {@link PortalView}.
 *
 * @param portal The portal to add to the internal registry.
 * @returns Promise containing the Portal that was added to the registry.
 */
const addPortal = async portal => {
  return IONPortalsReactNative.addPortal(portal);
};
/**
 * Adds all portals to an internal registry. This or {@link addPortal} must be called before attempting to render a {@link PortalView}
 *
 * @param portals The portals to add to the internal registry.
 * @returns Promise containing the Portals that were added to the registry.
 */


exports.addPortal = addPortal;

const addPortals = async portals => {
  return IONPortalsReactNative.addPortals(portals);
};
/**
 * Gets a {@link Portal} previously registered via {@link addPortal} or {@link addPortals}.
 *
 * @param name The portal name to retrieve from the internal registry.
 * @returns Promise containing the registered {@link Portal}. If the {@link Portal} was not registered, the Promise will fail.
 */


exports.addPortals = addPortals;

const getPortal = async name => {
  return IONPortalsReactNative.getPortal(name);
};

exports.getPortal = getPortal;

/**
 * Configures LiveUpdates to cyrptographically verify the contents of the downloaded bundles.
 * This method must be called before any LiveUpdates are registered otherwise they will no longer be tracked.
 *
 * @param pathToKey The *relative* path to the public key for verification.
 * This path should be the same relatibe to the main application bundle on iOS and the assets directory on Android.
 * @returns Promise<void>
 */
const enableSecureLiveUpdates = async pathToKey => {
  return IONPortalsReactNative.enableSecureLiveUpdates(pathToKey);
};
/**
 * Syncs a single live update.
 *
 * @param appId The AppFlow application ID to sync.
 * @returns A Promise<LiveUpdate>. A failure should result in a {@link LiveUpdateError}.
 */


exports.enableSecureLiveUpdates = enableSecureLiveUpdates;

const syncOne = async appId => {
  return IONPortalsReactNative.syncOne(appId);
};
/**
 * Syncs many live updates.
 *
 * @param appIds The AppFlow application IDs to sync.
 * @returns Promise<SyncResults>
 */


exports.syncOne = syncOne;

const syncSome = async appIds => {
  return IONPortalsReactNative.syncSome(appIds);
};
/**
 * Syncs all registered LiveUpdates
 * @returns Promise<SyncResults>
 */


exports.syncSome = syncSome;

const syncAll = async () => {
  return IONPortalsReactNative.syncAll();
};

exports.syncAll = syncAll;
//# sourceMappingURL=index.js.map